@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2006-2016. All Rights Reserved.
+%% Copyright Ericsson AB 2006-2017. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -26,7 +26,8 @@
 
 -export([prepare_tests/1, prepare_tests/2, 
 	 collect_tests_from_list/2, collect_tests_from_list/3,
-	 collect_tests_from_file/2, collect_tests_from_file/3]).
+	 collect_tests_from_file/2, collect_tests_from_file/3,
+         get_tests/1]).
 
 -export([testspec_rec2list/1, testspec_rec2list/2]).
 
@@ -803,6 +804,31 @@ list_nodes(#testspec{nodes=NodeRefs}) ->
     lists:map(fun({_Ref,Node}) -> Node end, NodeRefs).		      
 
 
+%%%-----------------------------------------------------------------
+%%% Parse the given test specs and return the complete set of specs
+%%% and tests to run/skip.
+%%% [Spec1,Spec2,...] means create separate tests per spec
+%%% [[Spec1,Spec2,...]] means merge all specs into one
+-spec get_tests(Specs) -> {ok,[{Specs,Tests}]} | {error,Reason} when
+      Specs :: [string()] | [[string()]],
+      Tests :: {Node,Run,Skip},
+      Node :: atom(),
+      Run :: {Dir,Suites,Cases},
+      Skip :: {Dir,Suites,Comment} | {Dir,Suites,Cases,Comment},
+      Dir :: string(),
+      Suites :: atom | [atom()] | all,
+      Cases :: atom | [atom()] | all,
+      Comment :: string(),
+      Reason :: term().
+
+get_tests(Specs) ->
+    case collect_tests_from_file(Specs,true) of
+        Tests when is_list(Tests) ->
+            {ok,[{S,prepare_tests(R)} || {S,R} <- Tests]};
+        Error ->
+            Error
+    end.
+
 %%     -----------------------------------------------------
 %%   /                                                       \
 %%  |  When adding test/config terms, remember to update      |
@@ -1132,6 +1158,11 @@ handle_data(verbosity,Node,VLvls,_Spec) when is_list(VLvls) ->
     VLvls1 = lists:map(fun(VLvl = {_Cat,_Lvl}) -> VLvl;
 			  (Lvl) -> {'$unspecified',Lvl} end, VLvls),
     [{Node,VLvls1}];
+handle_data(multiply_timetraps,Node,Mult,_Spec) when is_integer(Mult) ->
+    [{Node,Mult}];
+handle_data(scale_timetraps,Node,Scale,_Spec) when Scale == true;
+                                                   Scale == false ->
+    [{Node,Scale}];
 handle_data(silent_connections,Node,all,_Spec) ->
     [{Node,[all]}];
 handle_data(silent_connections,Node,Conn,_Spec) when is_atom(Conn) ->
@@ -1150,6 +1181,8 @@ should_be_added(Tag,Node,_Data,Spec) ->
 	Tag == label;        Tag == auto_compile;
 	Tag == abort_if_missing_suites;
 	Tag == stylesheet;   Tag == verbosity;
+        Tag == multiply_timetraps;
+        Tag == scale_timetraps;
 	Tag == silent_connections ->
 	    lists:keymember(ref2node(Node,Spec#testspec.nodes),1,
 			    read_field(Spec,Tag)) == false;
